@@ -1,22 +1,20 @@
 package Catalyst::Plugin::Session::Store::FastMmap;
 
 use strict;
-use base qw/
-    Class::Accessor::Fast 
-    Class::Data::Inheritable 
-    Catalyst::Plugin::Session::Store/;
+use Moose;
+use MRO::Compat;
+use namespace::clean -except => 'meta';
 
-use NEXT;
+with 'Catalyst::ClassData';
+with 'MooseX::Emulate::Class::Accessor::Fast';
+extends 'Catalyst::Plugin::Session::Store';
 
-BEGIN {
-    require Cache::FastMmap            if $^O ne 'MSWin32';
-    require Cache::FastMmap::WithWin32 if $^O eq 'MSWin32';
-}
+use Cache::FastMmap;
 use Path::Class     ();
 use File::Spec      ();
 use Catalyst::Utils ();
 
-our $VERSION = '0.05';
+our $VERSION = '0.16';
 
 __PACKAGE__->mk_classdata(qw/_session_fastmmap_storage/);
 
@@ -27,11 +25,13 @@ Catalyst::Plugin::Session::Store::FastMmap - FastMmap session storage backend.
 =head1 SYNOPSIS
 
     use Catalyst qw/Session Session::Store::FastMmap Session::State::Foo/;
-    
-    MyApp->config->{session} = {
-        expires => 3600,
-        storage => '/tmp/session'
-    };
+
+    MyApp->config(
+        'Plugin::Session' => {
+            expires => 3600,
+            storage => '/tmp/session'
+        },
+    );
 
     # ... in an action:
     $c->session->{foo} = 'bar'; # will be saved
@@ -71,7 +71,8 @@ sub get_session_data {
 
 sub store_session_data {
     my ( $c, $sid, $data ) = @_;
-    $c->_session_fastmmap_storage->set( $sid, $data );
+    $c->_session_fastmmap_storage->set( $sid, $data )
+        or Catalyst::Exception->throw("store_session: data too large");
 }
 
 sub delete_session_data {
@@ -85,7 +86,7 @@ sub get_and_set_session_data {
     my ( $c, $sid, $sub ) = @_;
     $c->_session_fastmmap_storage->get_and_set( $sid, sub {
         my ( $key, $data ) = @_;
-        my $new = $sub->( $key, $data ); 
+        my $new = $sub->( $key, $data );
         return $new;
     });
 }
@@ -99,12 +100,12 @@ Sets up the session cache file.
 sub setup_session {
     my $c = shift;
 
-    $c->NEXT::setup_session(@_);
+    $c->maybe::next::method(@_);
 
     my $tmpdir = Catalyst::Utils::class2tempdir($c)
       || Catalyst::Exception->throw("Can't determine tempdir for $c");
 
-    my $file = $c->config->{session}{storage} ||=
+    my $file = $c->_session_plugin_config->{storage} ||=
       File::Spec->catfile(    # Cache::FastMmap doesn't like Path::Class objects
         $tmpdir,
         "session_data",
@@ -113,18 +114,22 @@ sub setup_session {
     my $dir = Path::Class::file($file)->parent;
 
     unless (-d $dir) {
-        $dir->mkpath;
+        $dir->mkpath($c->debug);
     }
 
-    my $cfg = $c->config->{session};
+    if ($c->debug) {
+        $c->log->debug("Session Store file: $file");
+    }
+
+    my $cfg = $c->_session_plugin_config;
 
     $c->_session_fastmmap_storage(
         Cache::FastMmap->new(
             raw_values  => 0,
-            share_file  => $cfg->{storage},
+            share_file  => ($file . ''),  # force serialize in case it is a Path::Class object
             (
                 map { $_ => $cfg->{$_} }
-                  grep { exists $cfg->{$_} } qw/init_file cache_size/
+                  grep { exists $cfg->{$_} } qw/init_file cache_size page_size unlink_on_exit/
             ),
         )
     );
@@ -138,15 +143,21 @@ Very loaded sites with lots of data in the session hash may have old sessions
 expired prematurely, due to the LRU caching policy employed by
 L<Cache::FastMmap>. To get around this you can increase the C<cache_size>
 parameter, or switch session storage backends.
+L<Cache::FastMmap> defaults to around 5mb (89 * 64k).
 
 This is particularly inappropriate for use as a backend for e.g.
 L<Catalyst::Plugin::Session::PerUser>, for example.
 
-L<Cache::FastMmap> defaults to around 5mb (89 * 64k).
+As L<Cache::FastMmap> is not "thread-safe" (at least version 1.30 and before)
+therefore also this module does not work in multi-threaded environment.
+It is "fork-safe", however keep in mind that on Win32 the perl "fork" call is
+implemented as an emulation via threads - that is the reason why you cannot use
+this store for example when running you catalyst application on Win32 platform
+with L<Catalyst::Engine::HTTP::Prefork> engine.
 
 =head1 CONFIGURATION
 
-These parameters are placed in the hash under the C<session> key in the
+These parameters are placed in the hash under the C<Plugin::Session> key in the
 configuration hash.
 
 =over 4
@@ -168,6 +179,8 @@ UID of the process in the filename.
 
 =item cache_size
 
+=item unlink_on_exit
+
 See the L<Cache::FastMmap> documentation for the meaning of these keys. If
 these keys are not present L<Cache::FastMmap>'s defaults will be used.
 
@@ -182,15 +195,22 @@ L<Catalyst>, L<Catalyst::Plugin::Session>, L<Cache::FastMmap>.
 This module is derived from L<Catalyst::Plugin::Session::FastMmap> code, and
 has been heavily modified since.
 
-Andrew Ford
-Andy Grundman
-Christian Hansen
-Yuval Kogman, C<nothingmuch@woobling.org>
-Marcus Ramberg
-Sebastian Riedel
+  Andrew Ford
+  Andy Grundman
+  Christian Hansen
+  Yuval Kogman, <nothingmuch@woobling.org>
+  Marcus Ramberg
+  Sebastian Riedel
+  Tomas Doran, (t0m) <bobtfish@bobtfish.net>
 
 =head1 COPYRIGHT
 
+Copyright (c) 2005 - 2012
+the Catalyst::Plugin::Session::Store::FastMmap L</AUTHORS>
+as listed above.
+
+=head1 LICENSE
+
 This program is free software, you can redistribute it and/or modify it
 under the same terms as Perl itself.
 
